# Organize your work in projects

> **Objectives**: Create a project and a catalog, add this catalog to the project, and
> link a public catalog to the project with the OLP Command Line Interface (CLI).
>
> **Complexity**: Beginner
>
> **Time to complete**: 30 min
>
> **Prerequisites**: [Verify your credentials](../verify-credentials/README.md)
>
> **Source code**: <a href="organize-work-in-projects.zip" download target="_blank">Download</a>

This example demonstrates how to create projects, and how to manage project members
and resources using the [{{book.standard.cli}}]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/index.html).

A project is a collection of resources (catalogs, pipelines, and schemas) with controlled access.
The usage of any compute, storage, and transfer operations on a resource associated with the project
is automatically accounted for by project ID in the [Credit Usage report]({{book.filtered.portalUrlBase}}/profile/credit-usage) on the
{{book.standard.brandName}} platform. As a project
admin, you can grant other users, apps, and groups access to the project. Users with project access
rights can create and manage project resources.

To follow this tutorial, you need to have the [CLI installed]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/index.html#installer)
and have both [platform and repository credentials](../topics/get-credentials.html) set up on your local machine.

To verify that you set up your platform credentials correctly, make sure that the [Verify your credentials](../verify-credentials/README.md)
tutorial returns the expected result.

## Create a project

Give your project a unique identifier and name.

> #### Note
>
> The identifier and name of a project must be unique. The identifier cannot start or end with
> a hyphen and must contain from 4 to 16 lowercase alphanumeric characters including hyphens.

Replace `{{YOUR_PROJECT_ID}}` and `{{YOUR_PROJECT_NAME}}` with your own ID and name in
the command below and execute it:

```bash
[snippet](./configure-project.sh#project-create)
```

The {{book.standard.cli}} should return the following message:

```bash
Project {{YOUR_PROJECT_HRN}} has been created.
```

Write down `{{YOUR_PROJECT_HRN}}`, as you need it in the next steps for
the new project resources.

Notice that you can also create and manage your projects in the [{{book.standard.platformNameShort}} portal]({{book.filtered.portalUrlBase}}/projects).

To verify that your app and user have access to the project, execute the [`olp access list`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-access-commands.html#project-access-list) command as follows:

```bash
[snippet](./configure-project.sh#project-access-list)
```

If your command is successful, the CLI returns the following:

```bash
type           HRN                                                                         id                                           name
app            hrn:<partition>:account::<realm>:app/<AppId>                                <AppId>                                      <AppName>
user           hrn:<partition>:account::<realm>:user/<UserId>                              <UserId>                                     <UserName>
```

By default, the [`olp project create`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-commands.html#create-project) command grants project access both for the user who creates the project and for the app associated with the credentials. This behavior can be overridden with the `--skip-sharing-with-user` flag, which provides a way to not share the project with the user. Notice that without project access privileges, the user cannot view and manage the project in the {{book.standard.brandName}} portal.

You can also verify that an app and user have access to the project in the portal.
To do this, go to the [Project Manager]({{book.filtered.portalUrlBase}}/projects) portal section and select `{{YOUR_PROJECT_NAME}}` in the list. Verify that your user name and app name are listed in the "Access and permissions" section of the project details page.

To grant access to a project for a particular user, app, or group, use the [`olp project access grant`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-access-commands.html#project-access-grant) command as follows:

```bash
olp project access grant {{YOUR_PROJECT_HRN}} --app <appId> --user <uderId> --group <groupId>
```

## Create a catalog in the scope of the project

Give your catalog a unique identifier and name, for example,
`{{YOUR_USERNAME}}-sdii-tutorial`.

Create a configuration file for your catalog called `sdii-versioned.json` with the contents below and replace
`{{YOUR_CATALOG_ID}}` with your own identifier:

{% if book.filtered.RoW %}
```json
[snippet](./sdii-versioned.json)
```
{% else %}
```json
[snippet](./sdii-versioned-cn.json)
```
{% endif %}

To create a catalog, use the `olp catalog create` command. Replace `{{YOUR_CATALOG_ID}}` with your own catalog ID and
`{{YOUR_PROJECT_HRN}}` with the HRN of the project in the following command and execute it:

```bash
[snippet](./configure-project.sh#catalog-create)
```

> #### Note
> If a billing tag is required in your realm, update the config file by adding the `billingTags: ["YOUR_BILLING_TAG"]` property to the `layer` section.

Observe the `--scope` param, as the value of this parameter is equal to the project HRN. We specify this parameter to create a catalog in the scope of the project. In the same way, you should use this parameter with other operations such as update, show, and delete for resources associated with the project. For more details about scopes,
see the [Scopes]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/configuration.html#scopes) section of the CLI Developer Guide.

If your command is successful, the CLI returns the following:

```bash
Catalog {{YOUR_CATALOG_HRN}} has been created.
```

The HRN returned by this statement is what you use for all further CLI and
{{book.standard.productName}} operations to identify this catalog.

You can verify the expected catalog configuration with the [`olp catalog show`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/data/catalog-commands.html#catalog-show) command as follows:

```bash
[snippet](./configure-project.sh#catalog-show)
```

The CLI returns the values matching those in your JSON file. In particular,
the output that indicates that the versioned layer has been successfully created
and configured looks as follows:

```bash
...
layers
type      id                                      name
Versioned sdii-message-archive                    sdii-message-archive
...
```

You can also verify that the new catalog is shared with your project in the platform portal.
To do this, navigate to the [portal]({{book.filtered.portalUrlBase}}/projects) and select `{{YOUR_PROJECT_NAME}}` in the list.
Your new catalog should be listed in the "Resources" section of your project details page.

You can also inspect your new catalog configuration in the platform portal.
To do this, navigate to the [portal]({{book.filtered.portalUrlBase}}/data) and select `Simulated sensor data archive (From tutorial)`
from the list.

### Inspect catalog configuration

Notice that the fields in your JSON configuration correspond to the values displayed in the portal.

The _tags_ with which you have configured the catalog and layer appear in their
respective catalog listings in the [Data tab]({{book.filtered.portalUrlBase}}/data) under the catalog name and the layer listings within a single catalog. You can also search for all catalogs with a given tag, such as
["sensordata"]({{book.filtered.portalUrlBase}}/data/?searchText=sensordata&filters=&sort=created:desc&type=catalog).

Click your catalog and select the "sdii-message-archive" layer in the "Layers" section below.
For more information about layers and their types, see [Data User Guide]({{book.filtered.devGuidesUrlBase}}/data-user-guide/portal/layers/layers.html).

The JSON file above configures the layer to contain an archive of raw sensor
data messages. Each partition in this layer contains a single protobuf-encoded
sensor data message, which the JSON file indicates in the
`contentType` field. The `layerType` is `"versioned"` and the `volumeType`
is `"durable"`, indicating that the partition data is intended to persist.
If a client publishes new data to an existing partition, the old data remains
available in previous versions of the layer. The `partitioning` scheme is
`"generic"`, meaning that the messages are not spatially partitioned in the
catalog. Therefore, you can't bring up a visual representation of the partition
by clicking on the **Inspect** tab. Instead, the tab reports "No spatial
data."

Click the **Schema** tab to view the details of the schema that is bundled with the layer.
The HRN of the schema is also specified in the layer configuration file. As you learned in
[Verify your credentials](../verify-credentials/README.md), an HRN identifies
a resource such as a catalog or a schema. Client applications can use the
schema HRN to discover the associated artifacts for processing and decoding
data using the schema's specifications and language bindings. The platform portal itself
uses the schema artifacts to decode the partition contents and display their
values in the **Inspect** tab.

The newly created catalog is empty, so there are no partitions to decode.

Also, you can verify the catalog layer configuration with the [`olp catalog layer show`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/data/layer-commands.html#catalog-layer-show) command as follows:

```bash
[snippet](./configure-project.sh#layer-show)
```

## Link a public catalog to the project

We use public catalogs as the data source in further tutorials. Link the {{book.filtered.mapContentCatalog}} public catalog to your project to be able to access this catalog in the scope of the project.

To do this, use the [`olp project resource link`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-link-commands.html#project-resource-link) command.

Replace `{{PUBLIC_CATALOG_HRN}}` with {{book.filtered.mapContentCatalog}} and `{{YOUR_PROJECT_HRN}}` with the HRN of your project in the following command, then execute it:

```bash
[snippet](./configure-project.sh#link-public-catalog)
```

If your command is successful, the CLI returns the following message:
{% codesnippet language="bash" %}
Project resource {{book.filtered.mapContentCatalog}} has been linked.
{% endcodesnippet %}

To verify that the public catalog has been successfully linked to the project, execute the [`olp project resource list`]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-link-commands.html#project-resource-list) command as follows:

```bash
[snippet](./configure-project.sh#project-resource-list)
```

If your command is successful, the CLI returns the following message:
{% codesnippet language="bash" %}
Available resources in the project:

HRN                                                                                                Type           Relation
{{book.filtered.mapContentCatalog}}                                                                catalog        reference
{% raw %}{{YOUR_CATALOG_HRN}}{% endraw %}                                                                               catalog        home
{% endcodesnippet %}

Notice that there are other types of resources, like pipelines and schemas, which can be associated with the project, but in this tutorial we link only catalogs. Also, there are different types of relations between a project and the resources associated with it.

If a resource is created in the scope of the project, the relation type is "home". If you link a public catalog to the project, the relation type is "reference".

You can also verify that the public catalog is shared with your project in the platform portal.
To do this, navigate to the ["Projects" tab]({{book.filtered.portalUrlBase}}/projects) and find `{{YOUR_PROJECT_NAME}}` from the list.

Open the project and verify that the public catalog previously linked is present in the "Resources" section of your project details page.

You can also inspect the configuration of any catalog accessible to you on the platform portal.
To do this, navigate to [portal]({{book.filtered.portalUrlBase}}/data) and
find your public catalog by name.

## Further information

You can use the catalog you created in this tutorial as the output
catalog for the streaming application implemented in [Run a Stream Application Locally](../archive-stream/README.md).

For more details on projects, see [Manage Projects](https://developer.here.com/documentation/identity-access-management/dev_guide/topics/manage-projects.html).

For more details on {{book.standard.cli}} project access management commands,
see the relevant sections in [Project Access Commands]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-access-commands.html).

For more details on {{book.standard.cli}} project management commands,
see the relevant sections in [Project Commands]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/project/project-commands.html).

For more details on {{book.standard.cli}} catalog commands, see the relevant
[Catalog Commands]({{book.filtered.devGuidesUrlBase}}/open-location-platform-cli/user_guide/topics/data/catalog-commands.html).

If you want to see schema decoding in action over a non-spatially partitioned layer (that is, whose partitioning scheme is "generic"), you can inspect the [Administrative Locations layer]({{book.filtered.portalUrlBase}}/data/{{book.filtered.mapContentCatalog}}/administrative-locations/partition)
of the [{{book.standard.mapContent}} catalog]({{book.filtered.portalUrlBase}}/data/{{book.filtered.mapContentCatalog}}).
